import sys
import re
import argparse
import textwrap
from collections import namedtuple

class Create_R_Input:
    def __init__(self,input_file,input_file2,input_file3,output_file):     
        self.input_file = input_file
        self.input_file2 = input_file2
        self.input_file3 = input_file3
        self.output_file = output_file
        
    def create_r_input(self):

        line = self.input_file.readline()
        fields = line.split('\t')

        gcnt = int((len(fields)-1)/2)
        gnames = []
        for i in range(int(gcnt)):
            for j in range(0,len(fields[2*i+1])):
                if(fields[2*i+1][j]=='_'):
                    gnames.append(fields[2*i+1][:j])
                    break
                
        snames=[]
        cl=[]
        allele=[]
        scnt=0
        while True:
            line = self.input_file.readline()
            if not line:
                break
            fields = line.split('\t')
            for i in range(len(fields)):
                fields[i] = fields[i].replace('\n','')
            
            snames.append(fields[0])
            allele.append(fields[1:])
            cl.append(1)
            scnt = scnt+1

        line = self.input_file2.readline()
        fields = line.split('\t')

        gcnt2 = int((len(fields)-1)/2)
        if(gcnt2 != gcnt):
            print("Error: numbers of case and control genes are not matched.")
            sys.exit()
        
        for i in range(gcnt2):
            for j in range(0,len(fields[2*i+1])):
                if(fields[2*i+1][j]=='_'):
                    if(gnames[i] != fields[2*i+1][:j]):
                        print("Error: gene name of case and control are not matched.")
                        sys.exit()
                    break
                       
        while True:
            line = self.input_file2.readline()
            if not line:
                break
            fields = line.split('\t')
            for i in range(len(fields)):
                fields[i] = fields[i].replace('\n','')
            
            snames.append(fields[0])
            allele.append(fields[1:])
            cl.append(0)
            scnt = scnt+1
            
            
        D = [[["",0,[]]] for i in range(gcnt)]
        Dcnt = [0]*gcnt
        while True:
            line = self.input_file3.readline()
            if not line:
                break
            fields = line.split('\t')
            for i in range(len(fields)):
                fields[i] = fields[i].replace('\n','')

            if(fields[0][0] == '#'):
                if(fields[0][1:] in gnames):
                    k = gnames.index(fields[0][1:])
                    if(Dcnt[k]==0):
                        D[k][0][0] = fields[1]
                        D[k][0][1] = fields[2]
                    else:
                        D[k].append([fields[1],fields[2],[]])
                    Dcnt[k] = Dcnt[k]+1                    
            else:
                for j in range(0,len(fields[0])):
                    if(fields[0][j]=='*'):
                        tmpgname=fields[0][:j]
                        break
                if(tmpgname == gnames[k]):
                    D[k][Dcnt[k]-1][2].append([fields[0],fields[2]])
                 
        did = [0]*2

        headline = []
        headline.append("sample")
        headline.append("case")
        for g in range(gcnt):
            for k in range(Dcnt[g]):
                for i in range(2):
                    headline.append("_".join([gnames[g],D[g][k][0],str(i+1)]))

        ofile = open("".join([self.output_file,".aa.txt"]),'w')
        ofile.write("\t".join(headline))
        ofile.write("\n")
        for s in range(scnt):
            writeline = []
            writeline.append(snames[s])
            writeline.append(str(cl[s]))            
            for g in range(gcnt):                
                for k in range(Dcnt[g]):
                    a = D[g][k][1].split(",")
                                        
                    for i in range(2):               
                        hit = 0
                        for j in range(len(D[g][k][2])):
                            if(D[g][k][2][j][0] == allele[s][2*g+i]):
                                hit = 1
                                break                        
                        if(hit == 1):
                            writeline.append(D[g][k][2][j][1])
                        else:
                            writeline.append("".join(['.']*len(a)))
            ofile.write("\t".join(writeline))
            ofile.write("\n")
        ofile.close()

        ofile = open("".join([self.output_file,".pos.txt"]),'w')
        ofile.write("\t".join(["Domain","Position"]))
        ofile.write("\n")
        for g in range(gcnt):                
            for k in range(Dcnt[g]):
                writeline = []
                writeline.append("_".join([gnames[g],D[g][k][0]]))
                writeline.append(D[g][k][1])
                ofile.write("\t".join(writeline))
                ofile.write("\n")
                
                
        ofile.close()
                            
def main():
    parser = argparse.ArgumentParser(description='set G domain')
    parser.add_argument('case_result', help='Case typing result')
    parser.add_argument('control_result', help='Control typing result')
    parser.add_argument('aa_align_data', help='HLA amino acids data')
    parser.add_argument('output', help='Output file')

    args = parser.parse_args()

    input_file = open(args.case_result)
    input_file2 = open(args.control_result)
    input_file3 = open(args.aa_align_data)
        
    c = Create_R_Input(input_file,input_file2,input_file3,args.output)
    c.create_r_input()

    input_file.close()
    input_file2.close()
    input_file3.close()
    
        
if __name__ == '__main__': main()
