
glmHLA <- function(file1,file2){

X <- read.table(file1,sep="\t",head=T,quote="")
Y <- read.table(file2,head=T,sep="\t")

dcnt <- as.integer((ncol(X)-2)/2)

if(nrow(Y) != dcnt){
 print("Position file does not match with amino acids file.",quote=F)
 return(0)
}

n <- nrow(X)
glmhla <- list()

minp = 1.0
minpos = ""
minglm = {}

glmhla$dcnt <- dcnt
glmhla$p = {}
glmhla$pos = {}
glmhla$n = {}
glmhla$dname = c(1:dcnt)

for (d in 1:dcnt){

    dpos = 3+(d-1)*2
    nc <- nchar(as.character(X[1,3+(d-1)*2]))
    p = c(1:nc)
    aalist = list()

    s <- strsplit(names(X[dpos]),"_")
    name <- paste(s[[1]][1],s[[1]][2],sep="_")

    hit <- 0
    for(i in 1:dcnt){
      if(name == Y[i,1]){
      	hit = 1
	break
      }
    }

    if(hit == 0){
      print("Domain positions are not found in position file.",quote=F)
      return(0)
    }

    apos <- unlist(strsplit(as.character(Y[i,2]),","))

    if(nc != length(apos)){
      print("Position length is not matched.",quote=F)
      return(0)
    }

    for(pos in 1:nc){     

      f <- factor(cbind(substr(X[,dpos],pos,pos),substr(X[,dpos+1],pos,pos)))
      Caf <- factor(cbind(substr(X[X[,2]==1,dpos],pos,pos),substr(X[X[,2]==1,dpos+1],pos,pos)))
      Cof <- factor(cbind(substr(X[X[,2]==0,dpos],pos,pos),substr(X[X[,2]==0,dpos+1],pos,pos)))
      CaAA <- cbind(substr(X[X[,2]==1,dpos],pos,pos),substr(X[X[,2]==1,dpos+1],pos,pos))
      CoAA <- cbind(substr(X[X[,2]==0,dpos],pos,pos),substr(X[X[,2]==0,dpos+1],pos,pos))
      CaAA <- t(apply(CaAA,1,sort))
      CoAA <- t(apply(CoAA,1,sort))
      CaAA <- paste(CaAA[,1],CaAA[,2],sep="")
      CoAA <- paste(CoAA[,1],CoAA[,2],sep="")
      Camf <- factor(CaAA)
      Comf <- factor(CoAA)
      
      S <- summary(f)
      Sca <- summary(Caf)
      Sco <- summary(Cof)
      Scam <- summary(Camf)
      Scom <- summary(Comf)

      anames <- sort(levels(factor(c(names(Scam),names(Scom)))))
      Scaco <- matrix(0,nrow=length(anames),ncol=2)
      colnames(Scaco) <- c("Case","Control")
      rownames(Scaco) <- anames

      for(i in 1:length(Scam)){
      	for(j in 1:length(anames)){
	  if(names(Scam)[i] == rownames(Scaco)[j]){
	    Scaco[j,1] <- Scam[i]
	  }
	}
      }
      for(i in 1:length(Scom)){
      	for(j in 1:length(anames)){
	  if(names(Scom)[i] == rownames(Scaco)[j]){
	    Scaco[j,2] <- Scom[i]
	  }
	}
      }

      aalist <- c(aalist,list(Scaco))

      for(i in 1:length(S)){
      	if(names(S[i]) == names(which.max(Sco))){
	  ref = i
	}
      }     

      k = 0
      for (i in 1:length(S)){
    	if(names(S[i])=="."){
	 next
    	}
    	if(i==ref){
	 next
        }
    	k = k+1
      }
      
      if(k >= 1){
      	colna = c(1:k)
        A.A_ = matrix(nrow=n,ncol=k)
      	j = 1
        for (i in 1:length(S)){
          if(names(S[i])=="."){
	    next
          }
          if(i==ref){
	    next
          }
          A.A_[,j] = as.integer(substr(X[,dpos],pos,pos)==names(S[i])) + as.integer(substr(X[,dpos+1],pos,pos)==names(S[i]))
	  colna[j] <- names(S[i])
	  j = j+1
        }

	colnames(A.A_) <- colna
        myglm <- glm(case ~ A.A_,data=X,family="binomial")
        Sglm <- summary(myglm)
        p[pos] <- pchisq(Sglm$null.deviance-Sglm$deviance,df=Sglm$df.null-Sglm$df.residual,lower.tail=FALSE)

	if(p[pos] < minp){
	  minp = p[pos]
	  minpos =  paste(name,apos[pos],sep="_")
	  minglm = myglm
	}

      }else{
	p[pos] = 1.0
      }
    }

    glmhla$dname[d] <- name

    pname <- paste("p",name,sep=".")
    glmhla$p <- c(glmhla$p,list(p))
   
    posname <- paste("pos",name,sep=".")
    glmhla$pos <- c(glmhla$pos,list(apos))

    aaname <- paste("n",name,sep=".")
    glmhla$n <- c(glmhla$n,list(aalist))

  }
  glmhla$minp <- minp
  glmhla$minpos <- minpos
  glmhla$minglm <- minglm

  return(glmhla)
}


plot_glmhla <- function(glmhla){
  par(mai=c(2.0,1,0.2,0.5))

  ps = 18
  par(ps=ps)

  dcnt = glmhla$dcnt
  w = 120 
  maxx = w*(dcnt-1)+92

  allp = {}
  pos = {}

  for(i in 1:dcnt){
    allp <- c(allp,glmhla$p[[i]])
    pos <- c(pos,seq(1:length(glmhla$pos[[i]]))*(92/length(glmhla$pos[[i]]))+w*(i-1))
  }
  
  bon <- 0.05/(sum(allp < 1.0))
  yup = (-1*log(glmhla$minp,10) + 5)%/%5 * 5
  col11 <- rgb(0,1,1)

  plot(pos[allp<1.0],log(allp[allp<1.0],10),xlim=c(1,maxx),ylim=rev(c(-1*yup,0)),xaxt="n",xlab=NA,ylab=NA,bty="n",pch=20,col=col11,cex=1.4)

  par(new=T)

  col21 <- rgb(0,0,0.3)
  plot(pos[allp<1.0],log(allp[allp<1.0],10),xlim=c(1,maxx),ylim=rev(c(-1*yup,0)),xaxt="n",yaxt="n",xlab=NA,ylab=expression(paste(log[10],italic(" p"))),bty="n",pch=21,col=col21,cex=1.2,font=2)
  par(new=T)

  a <- which.min(allp)
  if(allp[a] < bon){
     plot(pos[a],log(allp[a],10),xlim=c(1,maxx),ylim=rev(c(-1*yup,0)),xaxt="n",yaxt="n",xlab=NA,ylab=NA,col="red",bty="n",pch=20,cex=2.5)
     }

  ps=14
  par(ps=ps)

  lables.data=c(1,92)
  for(i in 1:dcnt){
    axis(side=1,labels=F,at=c(w*(i-1)+1,w*(i-1)+92),lwd=1)
    mtext(1,at=c(w*(i-1)+1+6), text = "1", line=1.2,font=2)
    mtext(1,at=c(w*(i-1)+92-10), text = "92", line=1.2,font=2)
  }

  lx <- c(0,maxx)
  ly <- c(log(bon,10),log(bon,10))
  lines(lx,ly,lty=1,lwd=1.25,col="gray") 

　par(xpd = TRUE)
 
  for(i in 1:dcnt){
    #mtext(1,at=50+w*(i-1),text=glmhla$dname[i],srt=-315,line=2.5,font=2)
    text(y=yup/4,x=50+w*(i-1),labels=glmhla$dname[i],srt=-315,font=2)
  }

}

